# -*- coding: utf-8 -*-

from django.core.management.base import BaseCommand
from django.conf import settings
from django.utils import timezone

from guichet.models import Event, Registration
from guichet.utils import compute_event_stats
from paypal.standard.ipn.models import PayPalIPN

from datetime import timedelta, datetime
from decimal import Decimal
import json


class CustomEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime):
            return obj.isoformat()
        if isinstance(obj, Decimal):
            return float(obj)
        return json.JSONEncoder.default(self, obj)


class Command(BaseCommand):
    help = "Archivage des vieux évènements."

    def add_arguments(self, parser):
        parser.add_argument(
            "--dry-run",
            action="store_true",
            dest="dry_run",
            default=False,
            help="N'affiche que les actions qui devraient être effectuées",
        )

    def archive_event(self, event):
        event.archived_stats = CustomEncoder().encode(compute_event_stats(event))
        event.archived = True
        event.save()

        # delete logs
        event.logs.all().delete()

        # delete registrations IPN
        pks = Registration.objects.filter(ticket__event=event).values("pk")
        invoices = map((lambda x: settings.PAYPAL_INVOICE_PREFIX + str(x["pk"])), pks)
        PayPalIPN.objects.filter(invoice__in=invoices).delete()

        # delete ticket (and, with models.CASCADE Info_Sup, Option, Registration)
        event.tickets.all().delete()

        # last save because because
        event.save()

    def handle(self, *args, **options):
        time_after_end = timedelta(days=400)
        time_last_log = timedelta(days=7)
        events = []
        for event in Event.objects.filter(end_date__lt=timezone.now() - time_after_end, archived=False):
            if (
                not event.logs.all().exists()
                or event.logs.all().order_by("-date").first().date < timezone.now() - time_last_log
            ):
                events.append(event)

        if options["dry_run"]:
            if events:
                print("Évènement à archiver : ")
                for event in events:
                    print(" - {} ({})".format(event.name, event.pk))
                print("Relancer le script sans l'option dry-run pour archiver les évènements")
        else:
            for event in events:
                print("Archivage de {} ({})".format(event.name, event.pk))
                self.archive_event(event)
